@@ -4,7 +4,7 @@ import asyncio
 import os
 import json
 from datetime import datetime
-from typing import ClassVar, Dict
+from typing import ClassVar, Dict, List, Optional, Any
 
 from langchain_core.messages import SystemMessage
 
@@ -53,6 +53,9 @@ from botrun_flow_lang.langgraph_agents.agents.tools.create_mermaid_diagram impor
 from botrun_flow_lang.langgraph_agents.agents.tools.create_plotly_chart import (
     PlotlyChartTool,
 )
+from botrun_flow_lang.langgraph_agents.agents.tools.gemini_code_execution import (
+    GeminiCodeExecutionTool,
+)
 
 from botrun_flow_lang.langgraph_agents.agents.checkpointer.firestore_checkpointer import (
     AsyncFirestoreCheckpointer,
@@ -64,6 +67,12 @@ from botrun_flow_lang.utils.clients.rate_limit_client import RateLimitClient
 
 from dotenv import load_dotenv
 
+import copy  # 用於深拷貝 schema，避免意外修改原始對象
+
+# 假設 MultiServerMCPClient 和 StructuredTool 已經被正確導入
+from langchain.tools import StructuredTool  # 或 langchain_core.tools
+from langchain_mcp_adapters.client import MultiServerMCPClient
+
 load_dotenv()
 
 # logger = default_logger
@@ -119,7 +128,7 @@ class BotrunRateLimitException(Exception):
 def get_react_agent_model_name(model_name: str = ""):
     final_model_name = model_name
     if final_model_name == "":
-        final_model_name = "claude-3-7-sonnet-latest"
+        final_model_name = "claude-sonnet-4-20250514"
     logger.info(f"final_model_name: {final_model_name}")
     return final_model_name
 
@@ -157,7 +166,7 @@ def get_react_agent_model(model_name: str = ""):
             # 放到要用的時候才 init，不然loading 會花時間
             from langchain_openai import ChatOpenAI
 
-            openrouter_model_name = "anthropic/claude-3.7-sonnet"
+            openrouter_model_name = "anthropic/claude-sonnet-4"
             # openrouter_model_name = "openai/o4-mini-high"
             # openrouter_model_name = "openai/gpt-4.1"
             model = ChatOpenAI(
@@ -877,6 +886,14 @@ current_date_time 的回傳值，可以幫助判斷使用者查詢是關於過
     當使用 create_mermaid_diagram 工具時，你必須在回應中包含create_mermaid_diagram回傳的URL網址。
     請按照以下格式回應：
     [{mermaid_diagram_title}] ({mermaid_diagram_url})
+
+- gemini_code_execution:
+    使用此工具執行需要程式碼運算的任務，特別是：
+    - 複雜的數學計算
+    - 資料分析和處理
+    - 需要產生圖表或視覺化的任務
+    當使用者要求「計算」、「分析」、「處理資料」、「產生圖表」等任務時，優先使用此工具。
+    如果工具回傳包含圖片（images），請告知使用者已產生視覺化結果。
     <範例1>
     使用者提問：
     請幫我分析這個PDF的內容，根據開會決議，產出一個行動流程圖
@@ -990,12 +1007,172 @@ When comparing dates in user queries:
     """
 
 
-def create_react_agent_graph(
+def transform_anthropic_incompatible_schema(
+    schema_dict: dict,
+) -> tuple[dict, bool, str]:
+    """
+    轉換可能與 Anthropic 不相容的頂層 schema 結構。
+
+    Args:
+        schema_dict: 原始 schema 字典。
+
+    Returns:
+        tuple: (轉換後的 schema 字典, 是否進行了轉換, 附加到 description 的提示信息)
+    """
+    if not isinstance(schema_dict, dict):
+        return schema_dict, False, ""
+
+    keys_to_check = ["anyOf", "allOf", "oneOf"]
+    problematic_key = None
+    for key in keys_to_check:
+        if key in schema_dict:
+            problematic_key = key
+            break
+
+    if problematic_key:
+        print(f"  發現頂層 '{problematic_key}'，進行轉換...")
+        transformed = True
+        new_schema = {"type": "object", "properties": {}, "required": []}
+        description_notes = f"\n[開發者註記：此工具參數原使用 '{problematic_key}' 結構，已轉換。請依賴參數描述判斷必要輸入。]"
+
+        # 1. 合併 Properties
+        # 先加入頂層的 properties (如果存在)
+        if "properties" in schema_dict:
+            new_schema["properties"].update(copy.deepcopy(schema_dict["properties"]))
+        # 再合併來自 problematic_key 內部的 properties
+        for sub_schema in schema_dict.get(problematic_key, []):
+            if isinstance(sub_schema, dict) and "properties" in sub_schema:
+                # 注意：如果不同 sub_schema 有同名 property，後者會覆蓋前者
+                new_schema["properties"].update(copy.deepcopy(sub_schema["properties"]))
+
+        # 2. 處理 Required
+        top_level_required = set(schema_dict.get("required", []))
+
+        if problematic_key == "allOf":
+            # allOf: 合併所有 required
+            combined_required = top_level_required
+            for sub_schema in schema_dict.get(problematic_key, []):
+                if isinstance(sub_schema, dict) and "required" in sub_schema:
+                    combined_required.update(sub_schema["required"])
+            # 只保留實際存在於合併後 properties 中的 required 欄位
+            new_schema["required"] = sorted(
+                [req for req in combined_required if req in new_schema["properties"]]
+            )
+            description_notes += " 所有相關參數均需考慮。]"  # 簡單提示
+        elif problematic_key in ["anyOf", "oneOf"]:
+            # anyOf/oneOf: 只保留頂層 required，並在描述中說明選擇性
+            new_schema["required"] = sorted(
+                [req for req in top_level_required if req in new_schema["properties"]]
+            )
+            # 嘗試生成更具體的提示 (如果 sub_schema 結構簡單)
+            options = []
+            for sub_schema in schema_dict.get(problematic_key, []):
+                if isinstance(sub_schema, dict) and "required" in sub_schema:
+                    options.append(f"提供 '{', '.join(sub_schema['required'])}'")
+            if options:
+                description_notes += (
+                    f" 通常需要滿足以下條件之一：{'; 或 '.join(options)}。]"
+                )
+            else:
+                description_notes += " 請注意參數間的選擇關係。]"
+
+        print(
+            f"  轉換後 schema: {json.dumps(new_schema, indent=2, ensure_ascii=False)}"
+        )
+        return new_schema, transformed, description_notes
+    else:
+        return schema_dict, False, ""
+
+
+# --- Schema 轉換輔助函數 (從 _get_mcp_tools_async 提取) ---
+def _process_mcp_tools_for_anthropic(langchain_tools: List[Any]) -> List[Any]:
+    """處理 MCP 工具列表，轉換不相容的 Schema 並記錄日誌"""
+    if not langchain_tools:
+        logger.info("[_process_mcp_tools_for_anthropic] 警告 - 未找到任何工具。")
+        return []
+
+    logger.info(
+        f"[_process_mcp_tools_for_anthropic] --- 開始處理 {len(langchain_tools)} 個原始 MCP 工具 ---"
+    )
+
+    processed_tools = []
+    for mcp_tool in langchain_tools:
+        # 只處理 StructuredTool 或類似的有 args_schema 的工具
+        if not hasattr(mcp_tool, "args_schema") or not mcp_tool.args_schema:
+            logger.debug(
+                f"[_process_mcp_tools_for_anthropic] 工具 '{mcp_tool.name}' 沒有 args_schema，直接加入。"
+            )
+            processed_tools.append(mcp_tool)
+            continue
+
+        original_schema_dict = {}
+        try:
+            # 嘗試獲取 schema 字典 (根據 Pydantic 版本可能不同)
+            if hasattr(mcp_tool.args_schema, "model_json_schema"):  # Pydantic V2
+                original_schema_dict = mcp_tool.args_schema.model_json_schema()
+            elif hasattr(mcp_tool.args_schema, "schema"):  # Pydantic V1
+                original_schema_dict = mcp_tool.args_schema.schema()
+            elif isinstance(mcp_tool.args_schema, dict):  # 已經是字典？
+                original_schema_dict = mcp_tool.args_schema
+            else:
+                logger.warning(
+                    f"[_process_mcp_tools_for_anthropic] 無法獲取工具 '{mcp_tool.name}' 的 schema 字典 ({type(mcp_tool.args_schema)})，跳過轉換。"
+                )
+                processed_tools.append(mcp_tool)
+                continue
+
+            # 進行轉換檢查
+            logger.debug(
+                f"[_process_mcp_tools_for_anthropic] 檢查工具 '{mcp_tool.name}' 的 schema..."
+            )
+            new_schema_dict, transformed, desc_notes = (
+                transform_anthropic_incompatible_schema(
+                    copy.deepcopy(original_schema_dict)  # 使用深拷貝操作
+                )
+            )
+
+            if transformed:
+                mcp_tool.description += desc_notes
+                logger.info(
+                    f"[_process_mcp_tools_for_anthropic] 工具 '{mcp_tool.name}' 的描述已更新。"
+                )
+                if isinstance(mcp_tool.args_schema, dict):
+                    logger.debug(
+                        f"[_process_mcp_tools_for_anthropic] args_schema 是字典，直接替換工具 '{mcp_tool.name}' 的 schema。"
+                    )
+                    mcp_tool.args_schema = new_schema_dict
+                else:
+                    # 如果 args_schema 是 Pydantic 模型，直接修改可能無效或困難
+                    # 附加轉換後的字典可能是一種備選方案，但 Langchain/LangGraph 可能不直接使用它
+                    # 最好的方法是確保 get_tools 返回的工具的 args_schema 可以被修改，
+                    # 或者在創建工具時就使用轉換後的 schema。
+                    # 如果不能直接修改，附加屬性是一種標記方式，但可能需要在工具調用處處理。
+                    logger.warning(
+                        f"[_process_mcp_tools_for_anthropic] args_schema 不是字典 ({type(mcp_tool.args_schema)})，僅添加 _transformed_args_schema_dict 屬性到工具 '{mcp_tool.name}'。這可能不足以解決根本問題。"
+                    )
+                    setattr(mcp_tool, "_transformed_args_schema_dict", new_schema_dict)
+            processed_tools.append(mcp_tool)
+
+        except Exception as e_schema:
+            logger.error(
+                f"[_process_mcp_tools_for_anthropic] 處理工具 '{mcp_tool.name}' schema 時發生錯誤: {e_schema}",
+                exc_info=True,
+            )
+            processed_tools.append(mcp_tool)  # 保留原始工具
+
+    logger.info(
+        f"[_process_mcp_tools_for_anthropic] --- 完成工具處理，返回 {len(processed_tools)} 個工具 ---"
+    )
+    return processed_tools
+
+
+async def create_react_agent_graph(
     system_prompt: str = "",
     botrun_flow_lang_url: str = "",
     user_id: str = "",
     model_name: str = "",
     lang: str = LANG_EN,
+    mcp_client: Optional[MultiServerMCPClient] = None,  # <--- 接收客戶端實例
 ):
     """
     Create a react agent graph with optional system prompt
@@ -1006,12 +1183,14 @@ def create_react_agent_graph(
         user_id: User ID
         model_name: Model name to use
         lang: Language code (e.g., "en", "zh-TW")
+        mcp_config: MCP servers config
     """
 
     tools = [
         # 使用特定語言的工具
         CurrentDateTimeTool.for_language(lang),
         WebSearchTool.for_language(lang),
+        GeminiCodeExecutionTool.for_language(lang),  # 新增 Gemini Code Execution 工具
         compare_date_time,
         scrape,
     ]
@@ -1027,6 +1206,37 @@ def create_react_agent_graph(
         tools.append(PlotlyChartTool.for_language(lang))
         # print("tools============>", tools)
 
+    mcp_tools = []
+    if mcp_client:
+        logger.info("偵測到活動的 MCP Client，嘗試獲取 MCP 工具...")
+        try:
+            # 直接使用活動的 client 獲取工具
+            raw_mcp_tools = mcp_client.get_tools()  # 假設這是同步或異步方法
+            # 如果 get_tools 是異步的: raw_mcp_tools = await mcp_client.get_tools()
+
+            if raw_mcp_tools:
+                logger.info(f"從 MCP Client 獲取了 {len(raw_mcp_tools)} 個原始工具。")
+                # 處理 Schema (使用提取的輔助函數)
+                mcp_tools = _process_mcp_tools_for_anthropic(raw_mcp_tools)
+                if mcp_tools:
+                    tools.extend(mcp_tools)
+                    logger.info(f"已加入 {len(mcp_tools)} 個處理後的 MCP 工具。")
+                    logger.debug(
+                        f"加入的 MCP 工具名稱: {[tool.name for tool in mcp_tools]}"
+                    )
+                else:
+                    logger.warning("MCP 工具處理後列表為空。")
+            else:
+                logger.info("MCP Client 返回了空的工具列表。")
+
+        except Exception as e_get:
+            logger.error(
+                f"從 MCP Client 獲取或處理工具時發生錯誤: {e_get}", exc_info=True
+            )
+            # 即使出錯，也可能希望繼續執行（不帶 MCP 工具）
+    else:
+        logger.info("未提供 MCP Client，跳過 MCP 工具。")
+
     lang_specific_prompt = (
         zh_tw_system_prompt if lang == LANG_ZH_TW else en_system_prompt
     )
